/**
 * <a href="http://servicemix.org">ServiceMix: The open source ESB</a> 
 * 
 * Copyright 2005 RAJD Consultancy Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/

package org.servicemix.jbi.framework;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;
import javax.jbi.JBIException;
import javax.jbi.management.DeploymentException;
import javax.jbi.management.InstallerMBean;
import javax.management.JMException;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanOperationInfo;
import javax.management.ObjectName;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.servicemix.jbi.component.ComponentContextImpl;
import org.servicemix.jbi.container.EnvironmentContext;
import org.servicemix.jbi.container.JBIContainer;
import org.servicemix.jbi.deployment.ClassPath;
import org.servicemix.jbi.deployment.Component;
import org.servicemix.jbi.deployment.Descriptor;
import org.servicemix.jbi.deployment.InstallationDescriptorExtension;
import org.servicemix.jbi.deployment.SharedLibrary;
import org.servicemix.jbi.deployment.SharedLibraryList;
import org.servicemix.jbi.management.AttributeInfoHelper;
import org.servicemix.jbi.management.BaseLifeCycle;
import org.servicemix.jbi.management.ManagementContext;
import org.servicemix.jbi.management.OperationInfoHelper;
import org.servicemix.jbi.management.ParameterHelper;
import org.servicemix.jbi.util.FileUtil;
import org.springframework.context.support.FileSystemXmlApplicationContext;
import EDU.oswego.cs.dl.util.concurrent.ConcurrentHashMap;
import EDU.oswego.cs.dl.util.concurrent.CopyOnWriteArraySet;
import EDU.oswego.cs.dl.util.concurrent.SynchronizedBoolean;

/**
 * Installation Service - installs/uninstalls archives
 * 
 * @version $Revision: 1.12 $
 */
public class InstallationService extends BaseLifeCycle implements FrameworkInstallationService {
    private static final Log log = LogFactory.getLog(InstallationService.class);
    private static final String DESCRIPTOR_FILE = "META-INF/jbi.xml";
    private JBIContainer container;
    private EnvironmentContext environmentContext;
    private ManagementContext managementContext;
    private Timer statsTimer = new Timer(true);
    private TimerTask timerTask;
    private boolean monitorInstallationDirectory = true;;
    private int monitorInterval = 10;
    private long lastMonitorTime = 0;
    private SynchronizedBoolean started = new SynchronizedBoolean(false);
    private Set installationSet = new CopyOnWriteArraySet();
    private ClassLoaderService classLoaderService = new ClassLoaderService();
    private Map installers = new ConcurrentHashMap();

    /**
     * Get Description
     * 
     * @return description of this item
     */
    public String getDescription() {
        return "installs/uninstalls Components";
    }

    /**
     * Load the installer for a new component from a component installation package.
     * 
     * @param installJarURL - URL locating a jar file containing a JBI Installable Component.
     * @return - the JMX ObjectName of the InstallerMBean loaded from installJarURL.
     */
    public synchronized ObjectName loadNewInstaller(String installJarURL) {
        ObjectName result = null;
        try {
            File tmpDir = unpackLocation(installJarURL);
            if (tmpDir != null) {
                Descriptor root = buildDescriptor(tmpDir);
                if (root != null && root.getComponent() != null) {
                    String componentName = root.getComponent().getIdentification().getName();
                    if (!installers.containsKey(componentName)) {
                        InstallerMBeanImpl installer = doInstallArchive(tmpDir, root);
                        if (installer != null) {
                            result = installer.getObjectName();
                            installers.put(componentName, installer);
                        }
                    }
                    else {
                        log.error("An installer already exists for " + componentName);
                    }
                }
                else {
                    log.error("Could not find Component from: " + installJarURL);
                }
            }
            else {
                log.warn("location: " + installJarURL + " isn't valid");
            }
        }
        catch (DeploymentException e) {
            log.error("Deployment failed", e);
        }
        return result;
    }

    /**
     * Load the InstallerMBean for a previously installed component.
     * 
     * @param aComponentName - the component name identifying the installer to load.
     * @return - the JMX ObjectName of the InstallerMBean loaded from an existing installation context.
     */
    public ObjectName loadInstaller(String aComponentName) {
        ObjectName result = null;
        InstallerMBeanImpl installer = (InstallerMBeanImpl) installers.get(aComponentName);
        if (installer != null) {
            result = installer.getObjectName();
        }
        return result;
    }

    /**
     * Unload a JBI Installable Component installer.
     * 
     * @param aComponentName - the component name identifying the installer to unload.
     * @param isToBeDeleted - true iff the component is to be deleted as well.
     * @return - true if the operation was successful, otherwise false.
     */
    public boolean unloadInstaller(String aComponentName, boolean isToBeDeleted) {
        InstallerMBeanImpl installer = (InstallerMBeanImpl) installers.remove(aComponentName);
        boolean result = installer != null;
        if (result && isToBeDeleted) {
            try {
                installer.uninstall();
            }
            catch (JBIException e) {
                log.warn("Uninstall failed", e);
            }
        }
        return result;
    }

    /**
     * Install a shared library jar.
     * 
     * @param aSharedLibURI - URI locating a jar file containing a shared library.
     * @return - the name of the shared library loaded from aSharedLibURI.
     */
    public String installSharedLibrary(String aSharedLibURI) {
        String result = "";
        try {
            File tmpDir = unpackLocation(aSharedLibURI);
            if (tmpDir != null) {
                Descriptor root = buildDescriptor(tmpDir);
                SharedLibrary sl = root.getSharedLibrary();
                if (sl != null) {
                    result = doInstallSharedLibrary(tmpDir, sl);
                }
            }
            else {
                log.warn("location: " + aSharedLibURI + " isn't valid");
            }
        }
        catch (DeploymentException e) {
            log.error("Deployment failed", e);
        }
        return result;
    }

    /**
     * Uninstall a shared library.
     * 
     * @param aSharedLibName - the name of the shared library to uninstall.
     * @return - true iff the uninstall was successful.
     */
    public boolean uninstallSharedLibrary(String aSharedLibName) {
        boolean result = false;
        classLoaderService.removeSharedLibrary(aSharedLibName);
        try {
            environmentContext.removeSharedLibraryDirectory(aSharedLibName);
            result = true;
        }
        catch (IOException e) {
            log.error("Failed to remove shared library directory");
        }
        return result;
    }

    /**
     * @return Returns the monitorInstallationDirectory.
     */
    public boolean isMonitorInstallationDirectory() {
        return monitorInstallationDirectory;
    }

    /**
     * @param monitorInstallationDirectory The monitorInstallationDirectory to set.
     */
    public void setMonitorInstallationDirectory(boolean monitorInstallationDirectory) {
        this.monitorInstallationDirectory = monitorInstallationDirectory;
    }

    /**
     * @return Returns the monitorInterval (number in secs)
     */
    public int getMonitorInterval() {
        return monitorInterval;
    }

    /**
     * @param monitorInterval The monitorInterval to set (in secs)
     */
    public void setMonitorInterval(int monitorInterval) {
        this.monitorInterval = monitorInterval;
    }

    /**
     * Initialize the Service
     * 
     * @param container
     * @throws DeploymentException
     */
    public void init(JBIContainer container) throws DeploymentException {
        this.container = container;
        this.environmentContext = container.getEnvironmentContext();
        this.managementContext = container.getManagementContext();
        buildState();
    }

    /**
     * Start the item.
     * 
     * @exception javax.jbi.JBIException if the item fails to start.
     */
    public void start() throws javax.jbi.JBIException {
        super.start();
        if (started.commit(false, true)) {
            scheduleDirectoryTimer();
        }
    }

    /**
     * Stop the item. This suspends current messaging activities.
     * 
     * @exception javax.jbi.JBIException if the item fails to stop.
     */
    public void stop() throws javax.jbi.JBIException {
        if (started.commit(true, false)) {
            super.stop();
            if (timerTask != null) {
                timerTask.cancel();
            }
        }
    }

    /**
     * Shut down the item. The releases resources, preparatory to uninstallation.
     * 
     * @exception javax.jbi.JBIException if the item fails to shut down.
     */
    public void shutDown() throws javax.jbi.JBIException {
        super.shutDown();
        if (timerTask != null) {
            timerTask.cancel();
        }
    }

    /**
     * Install an archive
     * 
     * @param location
     * @throws DeploymentException
     */
    public void install(String location) throws DeploymentException {
        install(location, false);
    }

    /**
     * Install an archive
     * 
     * @param location
     * @param autoStart
     * @throws DeploymentException
     */
    public void install(String location, boolean autoStart) throws DeploymentException {
        File tmpDir = unpackLocation(location);
        if (tmpDir != null) {
            Descriptor root = buildDescriptor(tmpDir);
            if (root != null && root.getComponent() != null) {
                String componentName = root.getComponent().getIdentification().getName();
                if (!installers.containsKey(componentName)) {
                    InstallerMBeanImpl installer = doInstallArchive(tmpDir, root);
                    if (installer != null) {
                        try {
                            installer.install();
                        }
                        catch (JBIException e) {
                            throw new DeploymentException(e);
                        }
                        if (autoStart) {
                            ComponentNameSpace cns = new ComponentNameSpace(container.getName(), componentName,
                                    componentName);
                            LocalComponentConnector lcc = container.getRegistry().getLocalComponentConnector(cns);
                            if (lcc != null) {
                                ComponentMBean mbean = lcc.getComponentMBean();
                                if (mbean != null) {
                                    try {
                                        mbean.start();
                                    }
                                    catch (JBIException e) {
                                        String errStr = "Failed to start Component: " + componentName;
                                        log.error(errStr, e);
                                        throw new DeploymentException(e);
                                    }
                                    catch (IOException e) {
                                        String errStr = "Failed to start Component: " + componentName;
                                        log.error(errStr, e);
                                        throw new DeploymentException(e);
                                    }
                                }
                                else {
                                    log.warn("No ComponentMBean found for Component " + componentName);
                                }
                            }
                            else {
                                log.warn("No ComponentConnector found for Component " + componentName);
                            }
                        }
                    }
                }
                else {
                    log.warn("Component " + componentName + " is already installed");
                }
            }
            else {
                log.error("Could not find Component from: " + location);
            }
        }
        else {
            log.warn("location: " + location + " isn't valid");
        }
    }

    /**
     * Unpack a location into a temp file directory
     * 
     * @param location
     * @return tmp directory
     * @throws DeploymentException
     */
    protected File unpackLocation(String location) throws DeploymentException {
        File tmpDir = null;
        File tmpRoot = environmentContext.getTmpDir();
        try {
            File file = new File(location);
            if (!file.exists()) {
                // assume it's a URL
                try {
                    URL url = new URL(location);
                    String fileName = url.getFile();
                    if (fileName == null || (!fileName.endsWith(".zip") && !fileName.endsWith(".jar"))) {
                        throw new DeploymentException("Location: " + location + " is not an archive");
                    }
                    file = FileUtil.unpackArchive(url, tmpRoot);
                }
                catch (MalformedURLException e) {
                    throw new DeploymentException(e);
                }
            }
            if (FileUtil.archiveContainsEntry(file, DESCRIPTOR_FILE)) {
                tmpDir = FileUtil.createUniqueDirectory(tmpRoot, file.getName());
                FileUtil.unpackArchive(file, tmpDir);
                log.info("Unpacked archive " + location + " to " + tmpDir);
            }
        }
        catch (IOException e) {
            e.printStackTrace();
            log.error("I/O error installing archive", e);
            throw new DeploymentException(e);
        }
        return tmpDir;
    }

    /**
     * Build a Descriptor from a file archieve
     * 
     * @param directory
     * @return the Descriptor object
     * @throws DeploymentException
     */
    protected Descriptor buildDescriptor(File directory) throws DeploymentException {
        Descriptor root = null;
        File descriptor = new File(directory, DESCRIPTOR_FILE);
        if (descriptor.exists()) {
            try {
                FileSystemXmlApplicationContext context = new FileSystemXmlApplicationContext("file:///"
                        + descriptor.getAbsolutePath());
                root = (Descriptor) context.getBean("jbi");
            }
            catch (Throwable e) {
                log.error("Unable to read descriptor:", e);
                throw new DeploymentException(e);
            }
        }
        else {
            throw new DeploymentException("Descriptor file: " + descriptor + " doesn't exist");
        }
        return root;
    }

    private void mmonitorInstallationDirectory() {
        File installationDir = environmentContext.getInstallationDir();
        if (installationDir != null && installationDir.exists() && installationDir.isDirectory()) {
            File[] files = installationDir.listFiles();
            if (files != null) {
                for (int i = 0;i < files.length;i++) {
                    File file = files[i];
                    if ((file.getPath().endsWith(".jar") || file.getPath().endsWith(".zip"))
                            && (installationSet.add(file.getName()) || file.lastModified() > lastMonitorTime)) {
                        try {
                            log.info("Archive changed: installing " + file.getName() + " ...");
                            install(file.getAbsolutePath(), true);
                            log.info("Finished installation of archive:  " + file.getName());
                        }
                        catch (DeploymentException e) {
                            log.warn("Automatic install of " + file + " failed", e);
                        }
                    }
                }
            }
        }
        lastMonitorTime = System.currentTimeMillis();
    }

    private void scheduleDirectoryTimer() {
        if (isMonitorInstallationDirectory()) {
            if (timerTask != null) {
                timerTask.cancel();
            }
            timerTask = new TimerTask() {
                public void run() {
                    mmonitorInstallationDirectory();
                }
            };
            long interval = monitorInterval * 1000;
            statsTimer.scheduleAtFixedRate(timerTask, 0, interval);
        }
    }

    /**
     * Get an array of MBeanAttributeInfo
     * 
     * @return array of AttributeInfos
     * @throws JMException
     */
    public MBeanAttributeInfo[] getAttributeInfos() throws JMException {
        AttributeInfoHelper helper = new AttributeInfoHelper();
        helper.addAttribute(getObjectToManage(), "monitorInstallationDirectory",
                "Periodically monitor the Installation directory");
        helper.addAttribute(getObjectToManage(), "monitorInterval", "Interval (secs) before monitoring");
        return AttributeInfoHelper.join(super.getAttributeInfos(), helper.getAttributeInfos());
    }

    /**
     * Get an array of MBeanOperationInfo
     * 
     * @return array of OperationInfos
     * @throws JMException
     */
    public MBeanOperationInfo[] getOperationInfos() throws JMException {
        OperationInfoHelper helper = new OperationInfoHelper();
        ParameterHelper ph = helper.addOperation(getObjectToManage(), "loadNewInstaller", 1, "load a new Installer ");
        ph.setDescription(0, "installJarURL", "URL locating the install Jar");
        ph = helper.addOperation(getObjectToManage(), "loadInstaller", 1,
                "load installer for a previously installed component");
        ph.setDescription(0, "componentName", "Name of the Component");
        ph = helper.addOperation(getObjectToManage(), "unloadInstaller", 2, "unload an installer");
        ph.setDescription(0, "componentName", "Name of the Component");
        ph.setDescription(1, "isToBeDeleted", "true if component is to be deleted");
        ph = helper.addOperation(getObjectToManage(), "installSharedLibrary", 1, "Install a shared library jar");
        ph.setDescription(0, "sharedLibURI", "URI for the jar to be installed");
        ph = helper.addOperation(getObjectToManage(), "uninstallSharedLibrary", 1, "Uninstall a shared library jar");
        ph.setDescription(0, "sharedLibName", "name of the shared library");
        return OperationInfoHelper.join(super.getOperationInfos(), helper.getOperationInfos());
    }

    protected InstallerMBeanImpl doInstallArchive(File tmpDirectory, Descriptor descriptor) throws DeploymentException {
        InstallerMBeanImpl installer = null;
        Component component = descriptor.getComponent();
        if (component != null) {
            installer = doInstallComponent(tmpDirectory, component);
        }
        return installer;
    }

    protected String doInstallSharedLibrary(File tmpDirectory, SharedLibrary descriptor) throws DeploymentException {
        String result = null;
        if (descriptor != null) {
            try {
                result = descriptor.getIdentification().getName();
                File installationDir = environmentContext.createSharedLibraryDirectory(result);
                tmpDirectory.renameTo(installationDir);
                log.info("moved " + tmpDirectory + " to " + installationDir);
                classLoaderService.addSharedLibrary(installationDir, descriptor);
            }
            catch (Throwable e) {
                log.error("Deployment of Shared Library failed", e);
                // remove any files created for installation
                environmentContext.removeComponentRootDirectory(descriptor.getIdentification().getName());
                throw new DeploymentException(e);
            }
        }
        return result;
    }

    protected InstallerMBeanImpl doInstallComponent(File tmpDirectory, Component descriptor) throws DeploymentException {
        // move archive to Component directory
        InstallerMBeanImpl result = null;
        String name = descriptor.getIdentification().getName();
        File installationDir;
        try {
            installationDir = environmentContext.getInstallationDirectory(name);
        }
        catch (IOException e) {
            throw new DeploymentException(e);
        }
        tmpDirectory.renameTo(installationDir);
        log.info("moved " + tmpDirectory + " to " + installationDir);
        // create the Installation context
        result = initializeInstaller(installationDir, descriptor);
        return result;
    }

    private InstallerMBeanImpl initializeInstaller(File installationDir, Component descriptor)
            throws DeploymentException {
        InstallerMBeanImpl result = null;
        try {
            String name = descriptor.getIdentification().getName();
            InstallationContextImpl installationContext = new InstallationContextImpl();
            installationContext.setInstall(true);
            installationContext.setComponentName(name);
            installationContext.setComponentDescription(descriptor.getIdentification().getDescription());
            installationContext.setInstallRoot(installationDir);
            installationContext.setComponentClassName(descriptor.getComponentClassName());
            ClassPath cp = descriptor.getComponentClassPath();
            if (cp != null) {
                installationContext.setClassPathElements(cp.getPathElements());
            }
            // now build the ComponentContext
            installationContext.setContext(buildComponentContext(name));
            InstallationDescriptorExtension desc = descriptor.getDescriptorExtension();
            if (desc != null) {
                installationContext.setDescriptorExtension(desc.getDescriptorExtension());
            }
            installationContext.setBinding(descriptor.isBindingComponent());
            installationContext.setEngine(descriptor.isServiceEngine());
            // now we must initialize the boot strap class
            String bootstrapClassName = descriptor.getBootstrapClassName();
            ClassPath bootStrapClassPath = descriptor.getBootstrapClassPath();
            InstallationClassLoader bootstrapLoader = null;
            if (bootstrapClassName != null && bootstrapClassName.length() > 0) {
                boolean parentFirst = descriptor.isBootstrapClassLoaderDelegationParentFirst();
                bootstrapLoader = classLoaderService.buildClassLoader(installationDir, bootStrapClassPath
                        .getPathElements(), parentFirst);
            }
            SharedLibraryList[] lists = descriptor.getSharedLibraries();
            String componentClassName = descriptor.getComponentClassName();
            ClassPath componentClassPath = descriptor.getComponentClassPath();
            boolean parentFirst = descriptor.isComponentClassLoaderDelegationParentFirst();
            ClassLoader componentClassLoader = classLoaderService.buildClassLoader(installationDir, componentClassPath
                    .getPathElements(), parentFirst, lists);
            result = new InstallerMBeanImpl(container, installationContext, componentClassLoader, componentClassName,
                    bootstrapLoader, bootstrapClassName);
            // create an MBean for the installer
            ObjectName objectName = managementContext.createCustomComponentMBeanName(InstallerMBean.class.getName(),
                    name);
            result.setObjectName(objectName);
            managementContext.registerMBean(objectName, result, InstallerMBean.class,
                    "standard installation controls for a Component");
        }
        catch (Throwable e) {
            log.error("Deployment of Component failed", e);
            // remove any files created for installation
            environmentContext.removeComponentRootDirectory(descriptor.getIdentification().getName());
            throw new DeploymentException(e);
        }
        return result;
    }

    protected void buildState() throws DeploymentException {
        buildSharedLibs();
        buildComponents();
    }

    protected void buildSharedLibs() throws DeploymentException {
        // walk through shared libaries and add then to the ClassLoaderService
        File top = environmentContext.getSharedLibDir();
        if (top.exists() && top.isDirectory()) {
            // directory structure is sharedlibraries/<lib name>stuff ...
            File[] files = top.listFiles();
            if (files != null) {
                for (int i = 0;i < files.length;i++) {
                    if (files[i].isDirectory()) {
                        Descriptor root = buildDescriptor(files[i]);
                        if (root != null) {
                            SharedLibrary sl = root.getSharedLibrary();
                            if (sl != null) {
                                try {
                                    classLoaderService.addSharedLibrary(files[i], sl);
                                }
                                catch (MalformedURLException e) {
                                    log.error("Failed to initialize sharted library");
                                    throw new DeploymentException(e);
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    protected void buildComponents() throws DeploymentException {
        // walk through shared libaries and add then to the ClassLoaderService
        File top = environmentContext.getComponentsDir();
        if (top.exists() && top.isDirectory()) {
            // directory structure is components/<component name>/installation ...
            File[] files = top.listFiles();
            if (files != null) {
                for (int i = 0;i < files.length;i++) {
                    if (files[i].isDirectory()) {
                        try {
                            buildComponent(files[i]);
                        }
                        catch (DeploymentException e) {
                            log.error("Could not build Component: " + files[i].getName());
                            log.warn("Deleting Component directory: " + files[i]);
                            FileUtil.deleteFile(files[i]);
                        }
                    }
                }
            }
        }
    }

    protected void buildComponent(File componentDirectory) throws DeploymentException {
        try {
            File installationDirectory = environmentContext.getInstallationDirectory(componentDirectory.getName());
            Descriptor root = buildDescriptor(installationDirectory);
            if (root != null) {
                Component desc = root.getComponent();
                if (desc != null) {
                    String componentName = desc.getIdentification().getName();
                    if (!installers.containsKey(componentName)) {
                        InstallerMBean installer = initializeInstaller(installationDirectory, desc);
                        if (installer != null) {
                            if (installer != null) {
                                try {
                                    installer.install();
                                }
                                catch (JBIException e) {
                                    throw new DeploymentException(e);
                                }
                                // now get and set running state
                                ComponentNameSpace cns = new ComponentNameSpace(container.getName(), componentName,
                                        componentName);
                                LocalComponentConnector lcc = container.getRegistry().getLocalComponentConnector(cns);
                                if (lcc != null) {
                                    try {
                                        lcc.setRunningState();
                                    }
                                    catch (JBIException e) {
                                        log.warn("Failed to set running state for Component: " + componentName);
                                    }
                                }
                                else {
                                    log.error("Failed to get LocalComponentConnector for Component: " + componentName);
                                }
                            }
                            installers.put(componentName, installer);
                            log.info("Created installer for existing component:  " + componentName);
                        }
                    }
                    else {
                        log.warn("Component " + componentName + " is already installed");
                    }
                }
            }
        }
        catch (IOException e) {
            throw new DeploymentException(e);
        }
    }

    protected ComponentContextImpl buildComponentContext(String name) {
        ComponentNameSpace cns = new ComponentNameSpace(container.getName(), name, name);
        ComponentContextImpl context = new ComponentContextImpl(container, cns);
        return context;
    }
}
